@@ -3,15 +3,15 @@
 ################################################################################
 #                            TURN OFF WARNINGS                                 #
 ################################################################################
-
 import warnings
+import yaml
 from astropy.utils.exceptions import AstropyWarning
 
 warnings.simplefilter('ignore', UserWarning)
 warnings.simplefilter('ignore', FutureWarning)
 warnings.simplefilter('ignore', RuntimeWarning)     # warnings for the developer
 warnings.simplefilter('ignore', category=AstropyWarning)
-
+yaml.warnings({'YAMLLoadWarning': False})
 
 ################################################################################
 #                         PACKAGE GLOBAL VARIABLES                             #
