@@ -5,7 +5,8 @@ from astropy.table import Table
 
 from .. import effects as efs
 from ..effects.effects_utils import make_effect, get_all_effects
-from ..utils import table_to_rst, write_report
+from ..utils import write_report
+from ..reports.rst_utils import table_to_rst
 from .. import rc
 
 
@@ -112,7 +113,8 @@ class OpticalElement:
     @property
     def surfaces_list(self):
         _ter_list = [effect for effect in self.effects
-                     if isinstance(effect, (efs.SurfaceList, efs.TERCurve))]
+                     if isinstance(effect, (efs.SurfaceList, efs.FilterWheel,
+                                            efs.TERCurve))]
         return _ter_list
 
     @property
@@ -190,7 +192,16 @@ Global properties
 ::
 
 {}
+""".format(str(self),
+           rst_title_chars[0] * len(str(self)),
+           self.meta.get("object", "<unknown optical element>"),
+           self.meta.get("alias", "<unknown alias>"),
+           self.meta.get("description", "<no description>"),
+           rst_title_chars[1] * 17,
+           self.properties_str)
 
+        if len(self.list_effects()) > 0:
+            rst_str += """        
 Effects
 {}
 
@@ -201,14 +212,7 @@ Summary of Effects included in this optical element:
    
 {}
  
-""".format(str(self),
-           rst_title_chars[0] * len(str(self)),
-           self.meta.get("object", "<unknown optical element>"),
-           self.meta.get("alias", "<unknown alias>"),
-           self.meta.get("description", "<no description>"),
-           rst_title_chars[1] * 17,
-           self.properties_str,
-           rst_title_chars[1] * 7,
+""".format(rst_title_chars[1] * 7,
            "tbl:" + self.meta.get("name", "<unknown OpticalElement>"),
            table_to_rst(self.list_effects(), indent=4))
 
@@ -219,4 +223,3 @@ Summary of Effects included in this optical element:
         write_report(rst_str, filename, output)
 
         return rst_str
-
