@@ -66,6 +66,7 @@ class OpticalTrain:
 
     def __init__(self, cmds=None):
 
+        self._description = self.__repr__()
         self.cmds = cmds
         self.optics_manager = None
         self.fov_manager = None
@@ -74,6 +75,7 @@ class OpticalTrain:
         self.yaml_dicts = None
         self._last_source = None
 
+
         if cmds is not None:
             self.load(cmds)
 
@@ -119,7 +121,7 @@ class OpticalTrain:
         self.detector_arrays = [DetectorArray(det_list, **kwargs)
                                 for det_list in opt_man.detector_setup_effects]
 
-    def observe(self, orig_source, update=False, **kwargs):
+    def observe(self, orig_source, update=True, **kwargs):
         """
         Main controlling method for observing ``Source`` objects
 
@@ -199,13 +201,16 @@ class OpticalTrain:
         """
 
         hdus = []
-        for detector_array in self.detector_arrays:
+        for i, detector_array in enumerate(self.detector_arrays):
             dtcr_effects = self.optics_manager.detector_effects
             hdu = detector_array.readout(self.image_planes, dtcr_effects,
                                          **kwargs)
 
             if filename is not None and isinstance(filename, str):
-                hdu.writeto(filename, overwrite=True)
+                fname = filename
+                if len(self.detector_arrays) > 1:
+                    fname = str(i) + "_" + filename
+                hdu.writeto(fname, overwrite=True)
 
             hdus += [hdu]
 
@@ -218,10 +223,29 @@ class OpticalTrain:
             self.cmds.update(packages=self.default_yamls[0]["packages"])
         rc.__currsys__ = self.cmds
 
+
+    def shutdown(self):
+        '''Shut down the instrument.
+
+        This method closes all open file handles and should be called when the optical train
+        is no longer needed.
+        '''
+        for effect_name in self.effects['name']:
+            try:
+                self[effect_name]._file.close()
+            except AttributeError:
+                pass
+
+        self._description = "The instrument has been shut down."
+
+
     @property
     def effects(self):
         return self.optics_manager.list_effects()
 
+    def __str__(self):
+        return self._description
+
     def __getitem__(self, item):
         return self.optics_manager[item]
 
@@ -244,5 +268,3 @@ class OpticalTrain:
         # etc
         #   limiting magnitudes
         #
-
-
