@@ -3,6 +3,7 @@ Helper functions for ScopeSim
 """
 import math
 import os
+from pathlib import Path
 import sys
 import warnings
 from collections import OrderedDict
@@ -15,7 +16,6 @@ from astropy import units as u
 from astropy.io import fits
 from astropy.io import ascii as ioascii
 from astropy.table import Column, Table
-from astropy.table.pprint import TableFormatter
 
 from . import rc
 
@@ -564,18 +564,6 @@ def convert_table_comments_to_dict(tbl):
     return comments_dict
 
 
-def table_to_rst(tbl, indent=0):
-    tbl_fmtr = TableFormatter()
-    lines, outs = tbl_fmtr._pformat_table(tbl, max_width=-1, max_lines=-1)
-    lines[1] = lines[1].replace("-", "=")
-    lines = [lines[1]] + lines + [lines[1]]
-
-    indent = " " * indent
-    rst_str = indent + ("\n" + indent).join(lines)
-
-    return rst_str
-
-
 def change_table_entry(tbl, col_name, new_val, old_val=None, position=None):
 
     offending_col = list(tbl[col_name].data)
@@ -950,6 +938,7 @@ def write_report(text, filename=None, output=["rst"]):
                 out_text = out_text.decode("utf-8")
 
             suffix = {"rst": ".rst", "latex": ".tex"}[fmt]
-            from pathlib import Path
-            with open(Path(filename).stem + suffix, "w") as f:
+            fname = Path(filename)
+            fname = os.path.join(*fname.parts[:-1], fname.stem + suffix)
+            with open(fname, "w") as f:
                 f.write(out_text)
